package coupon

import (
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/coupon"
	"spaceofcheng/services/models/customer"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"time"

	"github.com/yl10/kit/guid"
)

// CouponServ 系统处理
type CouponServ struct {
	ctx     *utils.Context
	dao     *coupon.CouponDAO
	custDao *customer.CustomerDAO
}

// NewCouponServ 初始化
func NewCouponServ(ctx *utils.Context) *CouponServ {
	return &CouponServ{
		ctx:     ctx,
		dao:     coupon.NewCouponDAO(ctx),
		custDao: customer.NewCustomerDAO(ctx),
	}
}

// GetCouponList 获取优惠券列表
func (s *CouponServ) GetCouponList(caseID string, pagenavi ...int) ([]model.TaCoupon, int64, error) {
	if caseID == "" {
		return nil, 0, errors.New("未指定案场")
	}

	if err := utils.NewAuthEngine(s.ctx).CheckCase(caseID); err != nil {
		return nil, 0, err
	}

	limit := utils.GetPageNaviLimit(pagenavi...)
	filters := []string{
		"case_id='" + caseID + "'",
	}

	res, total, err := s.dao.GetCouponList(filters, limit)
	if err != nil {
		utils.LogError("查询优惠券列表失败: " + err.Error())
		return nil, 0, errors.New("查询优惠券列表失败")
	}

	return res, total, nil
}

// EditCoupon 编辑优惠券 , 没有则新增
func (s *CouponServ) EditCoupon(cp *coupon.CouponInfo) error {
	if cp.CouponId == "" {
		return s.SaveCoupon(cp)
	} else {
		return s.UpdateCoupon(cp)
	}
}

// SaveCoupon 保存优惠券
// 字段校验没有在此函数中设置
func (s *CouponServ) SaveCoupon(cp *coupon.CouponInfo) error {
	if cp.CaseId == "" {
		return errors.New("未指定案场")
	}

	cp.CouponId = guid.NewGUIDString()

	// 保存主信息
	cpMain := cp.TaCoupon
	if err := s.dao.SaveCoupon(&cpMain); err != nil {
		utils.LogError("保存优惠券失败: " + err.Error())
		return errors.New("保存优惠券失败")
	}

	// 保存图片
	if err := s.dao.UpdateImages(cp.Images, cp.CouponId); err != nil {
		utils.LogError("保存优惠券图片失败: " + err.Error())
		return errors.New("保存优惠券图片失败")
	}

	// 保存分享
	share := cp.Share
	share.CouponId = cp.CouponId
	cols := []string{"coupon_share_info", "use_rule", "use_instruction"}
	if err := s.dao.UpdateCouponShare(&share, cols); err != nil {
		utils.LogError("保存优惠券分享失败: " + err.Error())
		return errors.New("保存优惠券分享失败")
	}

	// 保存关联内容
	if err := s.dao.UpdateTarget(cp.Targets, cp.CouponId); err != nil {
		utils.LogError("保存优惠券关联内容失败: " + err.Error())
		return errors.New("保存优惠券关联内容失败")
	}

	// 保存规则
	if err := s.dao.UpdateRule(cp.Rules, cp.CouponId); err != nil {
		utils.LogError("保存优惠券规则失败: " + err.Error())
		return errors.New("保存优惠券规则失败")
	}

	return nil
}

// UpdateCoupon 更新优惠券
// 字段校验没有在此函数中设置
func (s *CouponServ) UpdateCoupon(cp *coupon.CouponInfo) error {
	if cp.CaseId == "" {
		return errors.New("未指定案场")
	}

	if cp.CouponId == "" {
		return errors.New("未指定优惠券")
	}

	// 更新主信息
	cpMain := cp.TaCoupon
	if err := s.dao.UpdateCoupon(&cpMain, []string{"coupon_name"}); err != nil {
		utils.LogError("更新优惠券失败: " + err.Error())
		return errors.New("更新优惠券失败")
	}

	// 更新图片
	if err := s.dao.UpdateImages(cp.Images, cp.CouponId); err != nil {
		utils.LogError("更新优惠券图片失败: " + err.Error())
		return errors.New("更新优惠券图片失败")
	}

	// 更新分享
	share := cp.Share
	share.CouponId = cp.CouponId
	cols := []string{"coupon_share_info", "use_rule", "use_instruction"}
	if err := s.dao.UpdateCouponShare(&share, cols); err != nil {
		utils.LogError("更新优惠券分享失败: " + err.Error())
		return errors.New("更新优惠券分享失败")
	}

	// 关联内容 规则 不允许更新
	return nil
}

// GiveCoupon 赠送卡券
func (s *CouponServ) GiveCoupon(couponID string, uids []string) error {
	// 当前用户
	userRaw := s.ctx.Get("user")
	if userRaw == nil {
		return errors.New("当前人员信息不正确, 请重试")
	}
	user := userRaw.(model.SysUser)

	cp, err := s.dao.GetCouponByID(couponID)
	if err != nil {
		utils.LogError("获取优惠券失败: " + err.Error())
		return errors.New("校验优惠券失败")
	}

	// 案场
	if err := utils.NewAuthEngine(s.ctx).CheckCase(cp.CaseId); err != nil {
		return err
	}

	// 优惠券必须状态正常
	if cp != nil || cp.CouponId == "" || cp.Status != models.STATUS_NORMAL {
		return errors.New("优惠券已失效或者不存在")
	}

	now := time.Now().Local()
	if cp.ValidDays <= 0 {
		if cp.EndDate.Before(now) {
			return errors.New("优惠券已失效")
		}
	}

	// 计算卡券有效期
	startDate := cp.StartDate
	endDate := cp.EndDate
	if cp.ValidDays > 0 {
		startDate = now
		hs := int64(cp.ValidDays * 24)
		endDate = now.Add(time.Hour * time.Duration(hs))
	}

	cps := make([]model.TaCustomerCoupon, 0)
	recs := make([]model.TaCouponGiveRecord, 0)
	for _, uid := range uids {
		// 获取客户信息
		cust, err := s.custDao.GetCustomerByID(uid)
		if err != nil {
			utils.LogError("查询客户信息失败: " + err.Error())
			return errors.New("校验客户信息失败")
		}

		// 检查用户是否已经有了该券
		custCPS, err := s.dao.CheckCustCoupon(uid, cp.CouponId, startDate, endDate)
		if err != nil {
			utils.LogError("校验客户优惠券失败: " + err.Error())
			return errors.New("校验客户优惠券失败")
		}

		if custCPS != nil && len(custCPS) > 0 {
			return errors.New("客户 " + cust.CustomerName + " 已经拥有该券")
		}

		// 放入我的卡券
		custCP := model.TaCustomerCoupon{
			CouponId:           cp.CouponId,
			CustomerCouponName: cp.CouponName,
			CustomerId:         uid,
			StartDate:          startDate,
			EndDate:            endDate,
			ReceiveDate:        now,
			CaseId:             cp.CaseId,
			OrgId:              cp.OrgId,
		}

		cps = append(cps, custCP)

		// 放入赠送记录
		rec := model.TaCouponGiveRecord{
			GiftType:   models.GIVE_GIFT_COUPON,
			GiftId:     cp.CouponId,
			GiftName:   cp.CouponName,
			FromId:     user.UserId,
			FromName:   user.UserName,
			ToId:       cust.CustomerId,
			ToName:     cust.CustomerName,
			CreateDate: now,
		}

		recs = append(recs, rec)
	}

	// 入库
	if err := s.dao.SaveCustomerCoupons(cps); err != nil {
		utils.LogError("保存客户优惠券失败: " + err.Error())
		return errors.New("保存客户优惠券失败")
	}

	if err := s.dao.SaveCouponRecoreds(recs); err != nil {
		utils.LogError("保存优惠券赠送记录失败: " + err.Error())
		return errors.New("保存优惠券赠送记录失败")
	}

	return nil
}
